//////////////////////////////////////////////////////////////////////////////
//	Cfx.js
//
//	Description:
//		JavaScript Class Framework 
//		(c) 2003 Chris Waldron  -- All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

var cfxJs = new _js();
var cfxClass = new _class();
var cfxDom = new _dom();

var Cfx = {}
Cfx.Js = cfxJs;
Cfx.Class = cfxClass;
Cfx.Dom = cfxDom;


//////////////////////////////////////////////////////////////////////////////
// Function extensions.
//////////////////////////////////////////////////////////////////////////////

// Properties.
Function.prototype.$className = "Function";
Function.prototype.$baseClass = null;
Function.prototype.InstanceOf = cfxClass.InstanceOf;
Function.prototype.Reflect = cfxClass.Reflect;

Function.prototype.BaseClass = function()	{ return this.$baseClass; };
Function.prototype.ClassName = function()	{ return cfxJs.FunctionName( cfxJs.ToFunction( this ) ); };
Function.prototype.GetClass = function()	{ return cfxJs.ToFunction( this ); };

Function.prototype.Delegate = function( arg )
{
	// Determine parameter type for delegate creation.
	return ( cfxJs.IsObject( this ) ) ? new Delegate( this, arg ) : new Delegate( arg, this );
};

//////////////////////////////////////////////////////////////////////////////
// Object extensions.
//////////////////////////////////////////////////////////////////////////////
Object.prototype.$className = "Object";
Object.prototype.$baseClass = null;
Object.prototype.BaseClass = Function.BaseClass;
Object.prototype.ClassName = Function.ClassName;
Object.prototype.GetClass = Function.GetClass;
Object.prototype.InstanceOf = cfxClass.InstanceOf;

Object.prototype.InitInstance = function()
{
	var thisClass = Object.InitInstance.caller;

	// Set instance class name.	
	if ( thisClass == this.constructor )
		this.$className = cfxJs.FunctionName( this.constructor );
		
	// Set instance base.
	if ( this.$baseClass == null )
		this.$baseClass = thisClass.$baseClass;

	// Set up base properties.
	thisClass.$baseClass.call( this );
};

Object.prototype.Method = function( func )
{
	this[cfxJs.FunctionName( func )] = func;
};

Object.prototype.Delegate = function( arg )
{
	return new Delegate( this, arg );
};


//////////////////////////////////////////////////////////////////////////////
// Date extensions.
//////////////////////////////////////////////////////////////////////////////
Date.prototype.$className = "Date";
Date.prototype.$baseClass = Object;
Date.prototype.$monthNames = 
	["January", "February", "March", "April", "May", "June",
	 "July", "August", "September", "October", "November", "December"];
   
Date.prototype.$monthDays = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

Date.prototype.$weekdayNames = 
	["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"];

Date.prototype.isLeapYear = function( year )
{
	year = ( year ) ? year : this.getYear();
	return ( (year % 400) == 0 ) ? true : 
		   ( (year % 100) == 0 ) ? false :
		   ( (year % 4)   == 0 ) ? true : false;
		   
};

Date.prototype.getMonthFullName = function( month )
{
	month = ( month ) ? month : this.getMonth();
	return this.$monthNames[month]; 
};

Date.prototype.getMonthShortName = function( month )
{
	month = ( month ) ? month : this.getMonth();
	return this.getMonthFullName( month ).substring( 0, 3 );
};

Date.prototype.getDaysInMonth = function( month, year )
{
	month = ( month ) ? month : this.getMonth();
	year = ( year ) ? year : this.getYear();
	var leap = ( ( month == 1 ) && this.isLeapYear( year ) ) ? 1 : 0;
	return this.$monthDays[month] + leap;
};

Date.prototype.getWeekdayFullName = function( weekday )
{
	weekday = ( weekday ) ? weekday : this.getDay();
	return this.$weekdayNames[weekday]; 
};

Date.prototype.getWeekdayShortName = function( weekday )
{
	weekday = ( weekday ) ? weekday : this.getDay();
	return this.getWeekdayFullName( weekday ).substring( 0, 3 );
};


//////////////////////////////////////////////////////////////////////////////
// Error extensions.
//////////////////////////////////////////////////////////////////////////////
Error.prototype.$className = "Error";
Error.prototype.$baseClass = Object;


//////////////////////////////////////////////////////////////////////////////
// RegExp extensions.
//////////////////////////////////////////////////////////////////////////////
RegExp.prototype.$className = "RegExp";
RegExp.prototype.$baseClass = Object;


//////////////////////////////////////////////////////////////////////////////
// String extensions.
//////////////////////////////////////////////////////////////////////////////
String.prototype.$className = "String";
String.prototype.$baseClass = Object;
String.prototype.trim = function()
{
	// Remove leading and trailing whitespace.
	return this.replace( /^\s+/, "" ).replace( /\s+$/, "" );
};


//////////////////////////////////////////////////////////////////////////////
//	Js
//////////////////////////////////////////////////////////////////////////////
function _js()
{
	//////////////////////////////////////////////////////////////////////////////
	//	IsArray( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.IsArray = function( obj )
	{
		return this.IsObject( obj ) && obj.constructor == Array;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsDefined( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.IsDefined = function( obj )
	{
	    return typeof( obj ) == 'undefined' ? false : true;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsDelegate( delegate )
	//////////////////////////////////////////////////////////////////////////////
	this.IsDelegate = function( delegate )
	{
		return ( delegate.$className == "Delegate" ) ? true : false;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsEmpty( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.IsEmpty = function( obj )
	{
		retval = true;
		if ( this.IsObject( obj ) )
		{
			for ( var item in obj )
			{
				retval = false;
				break;
            }
        }
        
		return retval;
	}

	//////////////////////////////////////////////////////////////////////////////
	//	IsFunction( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.IsFunction = function( obj )
	{
		return typeof( obj ) == 'function';
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsNull( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.IsNull = function( obj )
	{
		return this.IsObject( obj ) && ( obj == null );
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsNumeric( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.IsNumeric = function( obj )
	{
	    return typeof( obj ) == 'number' && isFinite( obj );
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsObject( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.IsObject = function( obj )
	{
		return typeof( obj ) == 'object';
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsString( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.IsString = function IsString( obj )
	{
		return typeof( obj ) == 'string';
	}


	//////////////////////////////////////////////////////////////////////////////
	//	FunctionName( func )
	//////////////////////////////////////////////////////////////////////////////
	this.FunctionName = function( func )
	{
		var funcName = null;
		if ( this.IsFunction( func ) )
		{
			var strFunc = func.toString();
			var reFuncName = new RegExp( /function\s+(\w+).*/g );
			var result = reFuncName.exec( strFunc );
			funcName = result[1];
		}
		
		return funcName;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	ToFunction( obj )
	//////////////////////////////////////////////////////////////////////////////
	this.ToFunction = function( obj )
	{
		return ( this.IsObject( obj ) ) ? obj.constructor : this.IsFunction( obj ) ? obj : null;
	}
}


//////////////////////////////////////////////////////////////////////////////
//	Class
//////////////////////////////////////////////////////////////////////////////
function _class()
{
	//////////////////////////////////////////////////////////////////////////////
	//	IsDefined( thisClass )
	//////////////////////////////////////////////////////////////////////////////
	this.IsDefined = function( thisClass )
	{
		return ( thisClass.$baseClass != null );
	}

	//////////////////////////////////////////////////////////////////////////////
	//	InstanceOf( thisClass, targClass )
	//////////////////////////////////////////////////////////////////////////////
	this.InstanceOf = function( thisClass, targClass )
	{
		var retval = false;

		// Set up arguments.
		if ( this.constructor != _class )
		{
			targClass = thisClass;
			thisClass = cfxJs.ToFunction( this );
		}

		// Convert arguments to functions.
		thisClass = cfxJs.ToFunction( thisClass );
		targClass = cfxJs.ToFunction( targClass );

		// Find instance of.
		if ( thisClass.$className == targClass.$className )
			retval = true;
		else if ( targClass == Object )
			retval = true;
		else if ( thisClass.$baseClass == null )
			retval = false;
		else 
			retval = cfxClass.InstanceOf( thisClass.$baseClass, targClass );
			
		// Return result.		
		return retval;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	New( thisClass )
	//////////////////////////////////////////////////////////////////////////////
	this.New = function( thisClass, baseClass )
	{
		if ( cfxClass.IsDefined( thisClass ) )
			return;
		
		// Inherit from base classes.
		if ( baseClass )
		{
			thisClass.prototype = new baseClass();
			thisClass.prototype.constructor = thisClass;
			thisClass.$baseClass = baseClass;
		}
		else
		{
			thisClass.$baseClass = Object;
			thisClass.$baseClass.$className = "Object";
		}
		
		// Initialize the class and return.
		thisClass.$className = cfxJs.FunctionName( thisClass );
		return;
	}
	
	
	//////////////////////////////////////////////////////////////////////////////
	//	Reflect( thisClass, commands )
	//
	//	Descriptions:
	//		Returns a string array of class member names.
	//
	//	Usage:
	//		Reflect( commands );
	//		Reflect( thisObj, commands );
	//		Reflect( thisClass, commands );
	//
	//		thisClass	--	Class to be reflected.
	//		commands	--	a command string.
	//						"a"		returns all members.
	//						"m"		returns method names.
	//						"p"		returns all property names.
	//						"v"		returns properties designated as variables.
	//
	//	Note:
	//		A variable is a property with its name not starting with a '$' or '_'.
	//
	//	Returns:
	//		A string array of the requested member names.
	//
	//	Examples:
	//		Reflect( class, "a" )  returns all member names.
	//		Reflect( class, "m" )  returns all method names.
	//		Reflect( class, "p" )  returns all property names.
	//		Reflect( class, "v" )  returns variable names
	//		Reflect( class, "mv" ) returns all method and variable names.
	//
	//////////////////////////////////////////////////////////////////////////////
	this.Reflect = function( thisObj, commands )
	{
		var options = "vpma";		// Ordered in reverse significance.
		var tally = 0;

		// Set up arguments.
		switch ( arguments.length )
		{
			case 0:
				commands = "a";
				thisObj = this;
				break;

			case 1:
				commands = arguments[0];
				thisObj = this;
				break;

			default:
				break;
		}

		// Determine the tally.
		for ( var ii = 0; ii < commands.length; ++ii )
			tally |= 1 << options.indexOf( commands.charAt( ii ) );
			
		// Allocate results.
		var results = new Array();
			
		// Obtain the methods if "a" or "m" is selected.
		if ( tally >= 6 || ( tally & 4 ) >= 4 || tally == 0 )
		{
			var thisClass = cfxJs.ToFunction( thisObj );

			for ( var name in thisClass.prototype )
			{
				if ( cfxJs.IsFunction( thisClass.prototype[name] ) )
					results[results.length] = name;
			}
		}
		
		// Obtain the methods if "a" or "p" is selected.
		if ( tally >= 6 || ( tally & 2 ) >= 2 || tally == 0 )
		{
			for ( var name in thisObj )
			{
				if ( !cfxJs.IsFunction( thisObj[name] ) )
					results[results.length] = name;
			}
		}
		
		// Obtain the variables if "v" is selected.
		else if ( ( tally & 1 ) == 1 )
		{
			for ( var name in thisObj )
			{
				if ( !cfxJs.IsFunction( thisObj[name] ) )
				{
					var firstChar = name.charAt(0);
					if ( firstChar != '$' && firstChar != '_' ) 
						results[results.length] = name;
				}
			}
		}
		
		return ( results.length ) ? results : null;
	}
}


//////////////////////////////////////////////////////////////////////////////
//	Dom
//	// Browser properties.
//	this.isIE = IsIE();
//	this.isNetscape = IsNetscape();
//	this.isMozilla = IsMozilla();
//	
//	// Document form function.
//	_dom.prototype.GetForm = GetForm;
//
//	// Document element functions.
//	_dom.prototype.FindElementById = FindElementById;
//	_dom.prototype.FindElementByName = FindElementByName;
//	_dom.prototype.FindElementSetById = FindElementSetById;
//	_dom.prototype.FindElementSetByName = FindElementSetByName;
//	_dom.prototype.GetElementById = GetElementById;
//	_dom.prototype.GetElementByName = GetElementByName;
//	_dom.prototype.GetElementTerms = GetElementTerms;
//
//	// Document image functions.
//	_dom.prototype.FindImage = FindImage;
//	_dom.prototype.FindImageSet = FindImageSet;
//	_dom.prototype.GetImage = GetImage;
//
//	// Document submit functions.
//	_dom.prototype.PostBack = PostBack;
//
//////////////////////////////////////////////////////////////////////////////
function _dom()
{
	// Browser properties.
	this.isIE = IsIE();
	this.isNetscape = IsNetscape();
	this.isMozilla = IsMozilla();
	
	//////////////////////////////////////////////////////////////////////////////
	//	IsIE()
	//////////////////////////////////////////////////////////////////////////////
	function IsIE()
	{
		var isApp = navigator.appName.indexOf( "Microsoft" ) != -1;
		
		var isVersion = ( arguments.length ) ?
					     navigator.appVersion.substring(0, 1).valueOf() >= arguments[0]
					     : true;
		
		return isApp && isVersion;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsMozilla()
	//////////////////////////////////////////////////////////////////////////////
	function IsMozilla()
	{
		var isApp = IsNetscape() && !document.all && document.getElementById;
		
		var isVersion = ( arguments.length ) ?
					     navigator.appVersion.substring(0, 1).valueOf() >= arguments[0]
					     : true;
					     
		return isApp && isVersion;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	IsNetscape()
	//////////////////////////////////////////////////////////////////////////////
	function IsNetscape()
	{
		var isApp = navigator.appName == "Netscape";
		
		var isVersion = ( arguments.length ) ?
					     navigator.appVersion.substring(0, 1).valueOf() >= arguments[0]
					     : true;
					     
		return isApp && isVersion;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	FindElementById( elementName )
	//////////////////////////////////////////////////////////////////////////////
	this.FindElementById = function( elementName )
	{
		var theForm = this.GetForm( arguments[1] );
		var nElements = theForm.elements.length;
		for ( var ii = 0; ii < nElements; ++ii )
		{
			var theElement = theForm.elements[ii];
			if ( theElement.id.indexOf( elementName ) != -1 ) 
				return theElement;
		}
		
		return null;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	FindElementByName( elementName )
	//////////////////////////////////////////////////////////////////////////////
	this.FindElementByName = function( elementName )
	{
		var theForm = this.GetForm( arguments[1] );
		var nElements = theForm.elements.length;
		for ( var ii = 0; ii < nElements; ++ii )
		{
			var theElement = theForm.elements[ii];
			if ( theElement.name.indexOf( elementName ) != -1 )
				return theElement;
		}
		
		return null;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	FindElementSetById( elementName )
	//////////////////////////////////////////////////////////////////////////////
	this.FindElementSetById = function( elementName )
	{
		var theForm = this.GetForm( arguments[1] );
		var nElements = theForm.elements.length;
		var elementSet = null;

		for ( var ii = 0; ii < nElements; ++ii )
		{
			var theElement = theForm.elements[ii];
			if ( theElement.id.indexOf( elementName ) != -1 )
			{
				if ( elementSet == null ) elementSet = new Array();
				elementSet[elementSet.length] = theElement;
			}
		}
		
		return elementSet;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	FindElementSetByName( elementName )
	//////////////////////////////////////////////////////////////////////////////
	this.FindElementSetByName = function( elementName )
	{
		var theForm = this.GetForm( arguments[1] );
		var nElements = theForm.elements.length;
		var elementSet = null;

		for ( var ii = 0; ii < nElements; ++ii )
		{
			var theElement = theForm.elements[ii];
			if ( theElement.name.indexOf( elementName ) != -1 )
			{
				if ( elementSet == null ) elementSet = new Array();
				elementSet[elementSet.length] = theElement;
			}
		}
		
		return elementSet;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	GetElementById( elementId )
	//////////////////////////////////////////////////////////////////////////////
	this.GetElementById = function( elementId )
	{
		return document.getElementById( elementId );
	}


	//////////////////////////////////////////////////////////////////////////////
	//	GetElementByName( elementName )
	//////////////////////////////////////////////////////////////////////////////
	this.GetElementByName = function( elementName )
	{
		var theForm = this.GetForm( arguments[1] );
		return ( theForm && theForm.elements[elementName] ) ? theForm.elements[elementName] : null;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	GetElementTerms
	//	
	//	Usage:
	//		GetElementTerms( argElement )
	//		GetElementTerms( argElement, splitChar )
	//////////////////////////////////////////////////////////////////////////////
	this.GetElementTerms = function( argElement, splitChar )
	{
		var elementId = null;
		var elementTerms = null;
		var splitCharDefault = "_";

		// Assume default split character is underscore;
		if ( !splitChar  ) splitChar = splitCharDefault;

		if ( cfxJs.IsObject( argElement ) )
			elementId = argElement.id;
		else if ( cfxJs.IsString( argElement ) )
			elementId = argElement;
		else
			throw new Error( "invalid element parameter" );

		// Perform string split if split character is not underscore.
		if ( splitChar != "_" )
			return thisElement.id.split( splitChar );
			
		// Split the element id with underscore as split character.
		while ( elementId != "" )
		{
			var results = elementId.match( /(_?[a-zA-Z\d]+_?)(\d+_?)?/ );
			if ( results != null )
			{
				var term = results[0];
				var termLen = term.length;
				if ( term.charAt( termLen - 1 ) == "_" )
					term = term.slice( 0, -1 );

				if ( elementTerms == null )
					elementTerms = new Array();

				elementTerms[elementTerms.length] = term;
				elementId = elementId.substr( termLen );
			}
		}

		return elementTerms;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	FindImage( imageName )
	//////////////////////////////////////////////////////////////////////////////
	this.FindImage = function( imageName )
	{
		var nImages = document.images.length;
		for( var ii = 0; ii < nImages; ++ii )
		{
			var theImage = document.images[ii];
			if ( theImage.id.indexOf( imageName ) != -1 )
				return theImage;
		}
		
		return null;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	FindImageSet( imageName )
	//////////////////////////////////////////////////////////////////////////////
	this.FindImageSet = function( imageName )
	{
		var nImages = document.images.length;
		var images = document.images;
		var imageSet = null;

		for ( var ii = 0; ii < nImages; ++ii )
		{
			if ( ( images[ii].id != null ) && ( images[ii].id.indexOf( imageName ) != -1 ) )
			{
				if ( imageSet == null ) imageSet = new Array();
					imageSet[imageSet.length] = images[ii];
			}
		}

		return imageSet;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	GetImage( imageId )
	//////////////////////////////////////////////////////////////////////////////
	this.GetImage = function( imageId )
	{
		var nImages = document.images.length;
		for( var ii = 0; ii < nImages; ++ii )
		{
			if ( document.images[imageId] )
				return document.images[imageId];
		}
		
		return null;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	GetForm( argForm )
	//////////////////////////////////////////////////////////////////////////////
	this.GetForm = function( argForm )
	{
		var theForm = null;

		if ( !argForm || argForm == null )
			theForm = document.forms[0];
		else if ( cfxJs.IsObject( argForm ) )
		{
			if ( argForm.$className == "FORM" )
				theForm = argForm;
		}
		else if ( cfxJs.IsNumeric( argForm ) )
			theForm = document.forms[argForm];
		else if ( cfxJs.IsString( argForm ) )
		{
			for ( var ii = 0; ii < documents.forms.length; ++ii )
			{
				if ( documents.forms[ii].id == objForm )
				{
					theForm = documents.forms[ii];
					break;
				}
			}
		}

		// Assign class name if we have a form.
		if ( theForm != null )
			 theForm.$className = "FORM";

		// Return result.
		return theForm;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	PostBack
	//	
	//	Usage:
	//		PostBack()
	//		PostBack( formIndex )
	//		PostBack( formIndex, eventTarget )
	//		PostBack( formIndex, eventTarget, eventArgument )
	//		PostBack( eventTarget )
	//		PostBack( eventTarget, eventArgument )
	//	
	//////////////////////////////////////////////////////////////////////////////
	this.PostBack = function()
	{
		var theForm = null;
		var eventTarget = null;
		var eventArgument = null;

		for ( var ii = 0; ii < arguments.length; ++ii )
		{
			if ( cfxJs.IsNumeric( arguments[ii] ) )
				theForm = Cfx.Dom.GetForm( arguments[0] );
			else if ( cfxJs.IsString( arguments[ii] ) )
			{
				if ( eventTarget == null )
					eventTarget = arguments[ii];
				else if ( eventArgument == null )
					eventArgument = arguments[ii];
				else
					throw new Error( "Invalid arguments" );
			}
			else
				throw new Error( "Invalid argument type" );
		}

		if ( theForm == null )
			theForm = Cfx.Dom.GetForm();

		if ( eventTarget == null )
			eventTarget = "";

		if ( eventArgument == null )
			eventArgument = "";

		if ( theForm.__EVENTTARGET )
			theForm.__EVENTARGET.value = eventTarget.split("$").join(":");

		if ( theForm.__EVENTARGUMENT )
			theForm.__EVENTARGUMENT.value = eventArgument;

		// Submit the form.
		theForm.submit();
	}
}		


//////////////////////////////////////////////////////////////////////////////
// Delegate object.
//////////////////////////////////////////////////////////////////////////////
function Delegate( obj, callback )
{
	if ( !cfxJs.IsObject( obj ) || !cfxJs.IsFunction( callback ) )
		throw new Error( "Invalid Delegate arguments" );

	var	m_obj = obj;
	var m_callback = callback;
	
	this.Invoke = function()
	{
		if ( m_obj && m_callback )
			return 	( arguments ) ?
				 m_callback.apply( m_obj, arguments ) : m_callback.apply( m_obj );
	}

	// Make Invoke function a Delegate type.
	this.Invoke.$className = "Delegate";
	this.Invoke.$baseClass = Function;
	this.Invoke.Invoke = this.Invoke;

	// Return the invocation method as the Delegate object.
	return this.Invoke;
}
