//////////////////////////////////////////////////////////////////////////////
//	CfxControl class
//		(c) 2003 Chris Waldron  -- All rights reserved.
//
//	Description:
//		Abstract base class for all cfx controls.
//
//////////////////////////////////////////////////////////////////////////////
function CfxControl()
{
	// Establish class and initialize instance.
	Cfx.Class.New( CfxControl );
	this.InitInstance();
	
	// Set methods.
	this.Method( ControlId );
	this.Method( GetElement );
	this.Method( GetItems );
	
	// Setup properties.
	this.items = null;
	this.name = null;
	this.parent = null;
	this.splitChar = "_";
	
	//////////////////////////////////////////////////////////////////////////
	// Class methods definitions
	//////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////
	//	GetElement( elementName )
	//////////////////////////////////////////////////////////////////////////
	function GetElement( elementName )
	{
		var theElement = null;

		if ( this.items != null )
		{
			for ( var ii = 0; ii < this.items.length; ++ii )
			{
				if ( this.items[ii].id.indexOf( elementName ) != -1 )
				{
					theElement = this.items[ii];
					break;
				}
			}
		}

		return theElement;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	GetItems( elementName )
	//
	//	Usage:
	//		GetItems( elementName )				:	Finds items for this control.
	//		GetItems( elementName, cfxControl )	:	Finds items for supplied control.
	//////////////////////////////////////////////////////////////////////////////
	function GetItems( elementName )
	{
		var thisCtrl = null;
		var items = null;

		switch ( arguments.length )
		{
			case 1:
				thisCtrl = this;
				break;

			case 2:
				thisCtrl = arguments[1];
				break;

			default:
				throw new Error( "Invalid arguments" );
		}

		// Determine whether the control is a form.
		var theForm = Cfx.Dom.GetForm( thisCtrl );
		if ( theForm != null )
		{
			// Obtain items from form.
			items = Cfx.Dom.FindElementSetById( elementName, theForm );
			var images = Cfx.Dom.FindImageSet( elementName, theForm );

			// Merge element and image sets
			if ( items == null )
				items = images;
			else if ( images != null )
			{
				for ( var ii = 0; ii < images.length; ++ii )
					items[items.length] = images[ii];
			}
		}
		else 
		{
			// Obtain items from control.
			for ( var ii = 0; ii < thisCtrl.items.length; ++ii )
			{
				if ( thisCtrl.items[ii].id.indexOf( elementName ) != -1 )
				{
					if ( items == null ) items = new Array();
					items[items.length] = thisCtrl.items[ii];
				}
			}
		}

		return items;
	}


	//////////////////////////////////////////////////////////////////////////
	//	ControlId : Builds control identifier id by walking up parent chain.
	//
	//	Usage:
	//		ControlId()
	//		ControlId( thisCtrl );
	//////////////////////////////////////////////////////////////////////////
	function ControlId()
	{
		var thisCtrl = ( arguments.length != 0 ) ? arguments[0] : this;

		if ( Cfx.Dom.GetForm( thisCtrl ) != null )
			return "";
			
		var result = this.ControlId( thisCtrl.parent );
		return ( result == "" ) ? thisCtrl.name : result + this.splitChar + thisCtrl.name;
	}
}
