//////////////////////////////////////////////////////////////////////////////
//	CfxCookie
//		(c) 2003 Chris Waldron  -- All rights reserved.
//
//	Description:
//		Cookie object.
//
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Cookie class.
//////////////////////////////////////////////////////////////////////////////
function CfxCookie()
{
	// Establish class and initialize instance.
	Cfx.Class.New( CfxCookie )
	this.InitInstance();
	
	// Set methods.
	this.Method( Delete );
	this.Method( Read );
	this.Method( Write );
	
	// Setup defaults.
	this._name = null;
	this._expiry = null;
	this._path = null;
	this._domain = null;
	this._secure = false;
	this._delimiters = ";=&=";
	
	// Argument settings.
	if ( arguments.length )
	{
		this._name = arguments[0];
		if ( arguments[1] ) this._expiry = arguments[1];
		if ( arguments[2] ) this._path = arguments[2];
		if ( arguments[3] ) this._domain = arguments[3];
		if ( arguments[4] ) this._secure = true;
	}

	//////////////////////////////////////////////////////////////////////////
	// Class methods definitions
	//////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////
	//	Delete
	//
	//	Description:
	//		Removes the cookie.
	//
	//////////////////////////////////////////////////////////////////////////
	function Delete()
	{
		var cookieSep = this._delimiters.charAt(0);
		var cookieEq = this._delimiters.charAt(1)

		// Set up cookie removal.
		var cookie = this._name + "=";
		cookie +=		
      		( ( this._path )	?	cookieSep + " path" + cookieEq : "" ) +
      		( ( this._domain )	?	cookieSep + " domain" + cookieEq : "" ) +
			cookieSep + " expires" + cookieEq + "Fri, 02-Jan-1970 00:00:00 GMT";

		// Remove the cookie.
		document.cookie = cookie;
	}
	

	//////////////////////////////////////////////////////////////////////////
	//	Read
	//
	//	Description:
	//		Read the cookie and constructs CfxCookie object variables.
	//
	//	Returns:
	//		true	:	Successful execution.
	//		false	:	Cannot find cookie.
	//	
	//////////////////////////////////////////////////////////////////////////
	function Read()
	{
		var cookieSep = this._delimiters.charAt(0);
		var cookieEq = this._delimiters.charAt(1)
		var varsSep = this._delimiters.charAt(2);
		var varsEq = this._delimiters.charAt(3);

		// Locate the cookie from the object name.
		var reCookie = new RegExp( this._name + cookieEq + "([^" + cookieSep + "]+)" );
		var results = reCookie.exec( document.cookie )
		if ( results == null )
			return false;

		// Construct cookie variables from the cookie value.
		var cookieValue = results[1];
		var varPairs = cookieValue.split( varsSep );
		for ( ii = 0; ii < varPairs.length; ++ii )
		{
			var variable = varPairs[ii].split( varsEq );
			this[variable[0]] = unescape( variable[1] );
		}

		return true;
	}


	//////////////////////////////////////////////////////////////////////////
	//	Write
	//
	//	Description:
	//		Writes the CfxCookie object variables into the cookie.
	//
	//////////////////////////////////////////////////////////////////////////
	function Write()
	{
		var strCookie = "";						// Initialize cookie string.
		var variables = this.Reflect( "v" );	// Obtain the variables.
		var cookieSep = this._delimiters.charAt(0);
		var cookieEq = this._delimiters.charAt(1)
		var varsSep = this._delimiters.charAt(2);
		var varsEq = this._delimiters.charAt(3);
		
		// Build cookie value from object variables.
		for ( var ii = 0; ii < variables.length; ++ii )
		{
			if ( strCookie != "" )	strCookie += varsSep;
			strCookie += variables[ii] + varsEq + escape( this[variables[ii]] );
		}
			
		// Complete the cookie string.
		var cookie = this._name + "=" + strCookie;
		cookie +=		
			( ( this._expiry )	?	cookieSep + " expires" + cookieEq + this._expiry.toGMTString() : "" ) +
      		( ( this._path )	?	cookieSep + " path" + cookieEq + this._path : "" ) +
      		( ( this._domain )	?	cookieSep + " domain" + cookieEq + this._domain : "" ) +
      		( ( this._secure )	?	cookieSep + " secure" : ""  );

		// Write the cookie.
		document.cookie = cookie;
	}
}
