//////////////////////////////////////////////////////////////////////////////
//	CfxHistoryOptions
//		(c) 2003 Chris Waldron  -- All rights reserved.
//
//	Description:
//		History options selection control.
//
//////////////////////////////////////////////////////////////////////////////
function CfxHistoryOptions()
{
	// Establish class and initialize instance.
	Cfx.Class.New( CfxHistoryOptions, CfxControl );
	this.InitInstance();
	
	// Set methods.
	this.Method( Initialize );
	this.Method( EnableControl );
	this.Method( OnRadioClick );
	this.Method( Validate );

	// Set properties.
	this.radioSet = null;
	
	// Initialize instance.
	if ( arguments.length )
	{
		this.name = arguments[0];
		this.parent = arguments[1];
		this.items = this.parent.GetItems( this.name );
		this.Initialize();
	}

	//////////////////////////////////////////////////////////////////////////
	// Class methods definitions
	//////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////
	// Initialize
	//////////////////////////////////////////////////////////////////////////
	function Initialize()
	{
		// Get radio controls.
		var radioSet = new CfxRadioSet( "radHistory", this );
		if ( radioSet != null )
		{
			// Set the property.
			this.radioSet = radioSet;

			// Clear status of the selectors
			var selection = new CfxSelect( "selByMonth", this );
			selection.ClearError();
			selection = new CfxSelect( "selByYear", this );
			selection.ClearError();

			// Set the initial radio selection.
			this.radioSet.ClickHandler( this.Delegate( OnRadioClick ) );
			this.radioSet.GetElement(0).click();
		}
		else
			throw new Error( "Missing radHistory in CfxHistoryOptions control" );
	}


	//////////////////////////////////////////////////////////////////////////
	//	EnableControl : Enable/disable associated radio controls.
	//////////////////////////////////////////////////////////////////////////
	function EnableControl( radElement, bEnable )
	{
		var radItems = this.GetItems( radElement.value );
		for ( var ii = 0; ii < radItems.length; ++ii )
		{
			if ( radItems[ii].type != "radio" )
				radItems[ii].disabled = !bEnable;
		}
	}


	//////////////////////////////////////////////////////////////////////////
	//	OnRadioClick : Receiver of radio onclick events.
	//////////////////////////////////////////////////////////////////////////
	function OnRadioClick( eventArgs )
	{
		// Create CfxEvent object and find the instance from the event.
		var cfxEvent = new CfxEvent( eventArgs );

		// Get the index of the selected radio control.
		var iSelected = this.radioSet.GetIndex( cfxEvent.Element() );

		// Enable the selected radio control; disable the rest.
		for ( ii = 0; ii < this.radioSet.Length(); ++ii )
		{
			var bEnable = ( ii == iSelected ) ? true : false;
			this.EnableControl( this.radioSet.GetElement(ii), bEnable );
		}
	}


	//////////////////////////////////////////////////////////////////////////
	//	Validate : Validate options.
	//////////////////////////////////////////////////////////////////////////
	function Validate()
	{
		var retval = false;
		var inputSet = new Array( this.radioSet.Length() );

		// Get the options for the radio buttons.
		for ( ii = 1; ii < inputSet.length; ++ii )
		{
			inputSet[ii] = new CfxInput();
			inputSet[ii].items = this.GetItems( this.radioSet.GetElement( ii ).value );
			inputSet[ii].ClearError();
		}
		
		// Validated selected option.
		var elSelected = this.radioSet.GetSelected();
		var iSelected = this.radioSet.GetIndex( elSelected );
		switch ( iSelected )
		{
			case 0:
				retval = true;
				break;
				
			case 1:
			case 2:
				var elDropDown = inputSet[iSelected].GetElement( "element" );
				if ( elDropDown.selectedIndex != 0 )
					retval = true 
				else
				{
					var msg1 = "Please select month";
					var msg2 = "Please select year";
					var msg = ( iSelected == 1 ) ? msg1 : msg2;
					inputSet[iSelected].AlertError( msg );
					retval = false;
				}
				break;
		}
		
		return retval;
	}
}
