//////////////////////////////////////////////////////////////////////////////
//	CfxRadioSet
//		(c) 2003 Chris Waldron  -- All rights reserved.
//
//	Description:
//		Control wrapper for a set of radio input elements.
//
//////////////////////////////////////////////////////////////////////////////
function CfxRadioSet()
{
	// Establish class and initialize instance.
	Cfx.Class.New( CfxRadioSet, CfxInput );
	this.InitInstance();
	
	// Set methods.
	this.Method( ClickHandler );
	this.Method( GetElement );
	this.Method( GetIndex );
	this.Method( GetItems );
	this.Method( GetSelected );
	this.Method( Length );
	
	// Setup instance data.
	if ( arguments.length )
	{
		this.name = arguments[0];
		this.parent = arguments[1];
		this.items = this.GetItems.call( this.parent, this.name );
	}

	//////////////////////////////////////////////////////////////////////////
	// Class methods definitions
	//////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////
	//	ClickHandler( delegate )
	//////////////////////////////////////////////////////////////////////////
	function ClickHandler( delegate )
	{
		if ( Cfx.Js.IsDelegate( delegate ) )
		{
			// Set radio items onclick event to the handler.
			for ( var ii = 0; ii < this.items.length; ++ii )
				this.items[ii].onclick = delegate;
		}
		else
			throw new Error( "ClickHandler argument is not a delegate" );
	}


	//////////////////////////////////////////////////////////////////////////
	//	GetElement( index )
	//////////////////////////////////////////////////////////////////////////
	function GetElement( index )
	{
		if ( index < 0 && index >= this.items.length )
			throw new Error( "Invalid index argument" );

		return this.items[index];
	}


	//////////////////////////////////////////////////////////////////////////
	//	GetIndex( theElement )
	//////////////////////////////////////////////////////////////////////////
	function GetIndex( theElement )
	{
		var index = -1;

		// Find the CfxRadioSet index of the radio element..
		for ( var ii = 0; ii < this.items.length; ++ii )
		{
			if ( this.items[ii].id == theElement.id )
			{
				index = ii;
				break;
			}
		}

		// return the index.
		return index;
	}


	//////////////////////////////////////////////////////////////////////////////
	//	GetItems
	//
	//	Usage:
	//		GetItems( elementName )	
	//////////////////////////////////////////////////////////////////////////////
	function GetItems( elementName )
	{
		var items = null;

		// Determine whether the control is a form.
		var theForm = Cfx.Dom.GetForm( this );
		if ( theForm != null )
		{
			// Obtain items from form.
			items = Cfx.Dom.FindElementSetByName( elementName, theForm );
		}
		else 
		{
			// Obtain items from control.
			for ( var ii = 0; ii < this.items.length; ++ii )
			{
				if ( this.items[ii].name.indexOf( elementName ) != -1 )
				{
					if ( items == null ) items = new Array();
					items[items.length] = this.items[ii];
				}
			}
		}

		return items;
	}


	//////////////////////////////////////////////////////////////////////////
	//	GetSelected()
	//////////////////////////////////////////////////////////////////////////
	function GetSelected()
	{
		var elSelected = null;
		
		for ( var ii = 0; ii < this.items.length; ++ii )
		{
			if ( this.items[ii].checked == true )
			{
				elSelected = this.items[ii];
				break;
			}
		}
		
		return elSelected;
	}
		

	//////////////////////////////////////////////////////////////////////////
	//	Length
	//////////////////////////////////////////////////////////////////////////
	function Length()
	{
		return this.items.length;
	}
}
