//////////////////////////////////////////////////////////////////////////////
//	CfxSelect
//		(c) 2003 Chris Waldron  -- All rights reserved.
//
//	Description:
//		Single dropdown selection control.
//
//////////////////////////////////////////////////////////////////////////////
function CfxSelect()
{
	// Establish class and initialize instance.
	Cfx.Class.New( CfxSelect, CfxInput );
	this.InitInstance();
	
	// Set methods.
	this.Method( Add );
	this.Method( Initialize );
	this.Method( IsEmpty );
	this.Method( Length );
	this.Method( Remove );
	this.Method( SelectedIndex );
	this.Method( Text );
	this.Method( Value );
	
	// Setup instance data.
	if ( arguments.length )
	{
		this.name = arguments[0];
		this.parent = arguments[1];
		this.items = this.parent.GetItems( this.name );
		this.Initialize();
	}


	//////////////////////////////////////////////////////////////////////////
	// Class methods definitions
	//////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////
	// Initialize
	//////////////////////////////////////////////////////////////////////////
	function Initialize()
	{
		this.element = this.GetElement( "element" );
		if ( this.element == null )
			throw new CfxError( this, "Missing 'element' in CfxSelect control" );

		this.ClearError();
	}


	//////////////////////////////////////////////////////////////////////////
	//	Add( element )
	//////////////////////////////////////////////////////////////////////////
	function Add( element )
	{
		this.element.options[this.element.options.length] = element;
	}


	//////////////////////////////////////////////////////////////////////////
	//	IsEmpty
	//////////////////////////////////////////////////////////////////////////
	function IsEmpty()
	{
		return ( this.element.options.length == 0 ) ? true : false;
	}


	//////////////////////////////////////////////////////////////////////////
	//	Length
	//////////////////////////////////////////////////////////////////////////
	function Length()
	{
		return this.element.options.length;
	}


	//////////////////////////////////////////////////////////////////////////
	//	Remove( index )
	//////////////////////////////////////////////////////////////////////////
	function Remove( index )
	{
		if ( document.all )
			this.element.options.remove( index );
		else if ( document.getElementById )
			this.element.options[index] = null;
	}


	//////////////////////////////////////////////////////////////////////////
	//	SelectedIndex
	//////////////////////////////////////////////////////////////////////////
	function SelectedIndex()
	{
		return this.element.selectedIndex;
	}


	//////////////////////////////////////////////////////////////////////////
	//	Text Accessor
	// 
	//	Usage:
	//		Text()					:	gets selected value
	//		Text( index )			:	gets value from index position.
	//		Text( index, text )		:	sets value at index position.
	//////////////////////////////////////////////////////////////////////////
	function Text()
	{
		switch ( arguments.length )
		{
			// getter.
			case 0:
				return this.element.options[this.element.selectedIndex].text;

			case 1:
				return this.element.options[arguments[0]].text;

			// setter.
			case 2:
				return this.element.options[arguments[0]].text = arguments[1];

			default:
				throw new Error( "invalid arguments" );
		}
	}


	//////////////////////////////////////////////////////////////////////////
	//	Value Accessor
	// 
	//	Usage:
	//		Value()					:	gets selected value
	//		Value( index )			:	gets value from index position.
	//		Value( index, value )	:	sets value at index position.
	//////////////////////////////////////////////////////////////////////////
	function Value()
	{
		switch ( arguments.length )
		{
			// getter.
			case 0:
				return this.element.options[this.element.selectedIndex].value;

			case 1:
				return this.element.options[arguments[0]].value;

			// setter.
			case 2:
				return this.element.options[arguments[0]].value = arguments[1];

			default:
				throw new Error( "invalid arguments" );
		}
	}
}
