//////////////////////////////////////////////////////////////////////////////
//	CfxText
//		(c) 2003 Chris Waldron  -- All rights reserved.
//
//	Description:
//		Text input control.
//
//////////////////////////////////////////////////////////////////////////////
function CfxText()
{
	// Establish class and initialize instance.
	Cfx.Class.New( CfxText, CfxInput );
	this.InitInstance();
	
	// Set methods.
	this.Method( Initialize );
	this.Method( IsEmpty );
	this.Method( IsLimit );
	this.Method( OnEnterKey );
	this.Method( SubmitHandler );
	this.Method( Validate );
	this.Method( Value );
	
	// Set properties.
	this.regExp = null;
	this.submit = null;
	
	// Initialize instance.
	if ( arguments.length )
	{
		this.name = arguments[0];
		this.parent = arguments[1];
		this.items = this.parent.GetItems( this.name );
		this.Initialize( arguments );
	}

	//////////////////////////////////////////////////////////////////////////
	// Class methods definitions
	//////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////
	// Initialize
	//////////////////////////////////////////////////////////////////////////
	function Initialize( args )
	{
		// Set regular expression.
		if ( args.length == 3 )
			this.regExp = args[2];

		// Set element.
		this.element = this.GetElement( "element" );
		if ( this.element == null )
			throw new CfxError( this, "Missing txtElement in CfxText control" );

		// Clear image.
		this.ClearError();
	}


	//////////////////////////////////////////////////////////////////////////
	//	IsEmpty
	//////////////////////////////////////////////////////////////////////////
	function IsEmpty()
	{
		var theText = this.element.value.toString();
		return ( theText.trim() == "" ) ? true : false;
	}


	//////////////////////////////////////////////////////////////////////////
	//	IsLimit
	//////////////////////////////////////////////////////////////////////////
	function IsLimit()
	{
		var theText = this.element.value.toString();
		return ( theText.trim().length > 255 ) ? true : false;
	}


	//////////////////////////////////////////////////////////////////////////
	// OnEnterKey()
	//////////////////////////////////////////////////////////////////////////
	function OnEnterKey( eventArgs )
	{
		var cfxEvent = new CfxEvent( eventArgs );
		return ( cfxEvent.IsEnterKey() ) ? this.submit : true;
	}


	//////////////////////////////////////////////////////////////////////////
	// SubmitHandler
	//////////////////////////////////////////////////////////////////////////
	function SubmitHandler( handler )
	{
		this.submit = handler;
		this.GetElement( "element" ).onkeypress = this.Delegate( OnEnterKey );
	}


	//////////////////////////////////////////////////////////////////////////
	//	Validate : Validate options.
	//////////////////////////////////////////////////////////////////////////
	function Validate()
	{
		var retval = false;

		if ( this.IsEmpty() )
			retval = ( this.regExp == null ) ? true : false;
		else if ( this.regExp == null )
			retval = true;
		else
		{
		    var elText = this.element;

			// Validate text field, and test for invalid character combinations.
			var reText = new RegExp( this.regExp );
			var theText = elText.value.toString();
			if ( ( reText.test( theText.trim() ) == true ) && ( theText.trim().indexOf("&#") == -1 ) )
			{
				this.ClearError();
				retval = true;
			}
			else
			{
				throw new CfxError( this, "Please remove invalid character(s) found in entry." );
			}
		}			

		return retval;
	}


	//////////////////////////////////////////////////////////////////////////
	// Value
	//////////////////////////////////////////////////////////////////////////
	function Value()
	{
		// setter.
		if ( arguments.length )
			this.element.value = arguments[0];

		// getter.
		return this.element.value.toString().trim();
	}
}
