//////////////////////////////////////////////////////////////////////////////
//	CfxWebFormOptions
//		(c) 2003 Chris Waldron  -- All rights reserved.
//
//	Description:
//		Container control for the web page options.
//
//	Usage:
//		CfxWebFormOptions( name )
//		CfxWebFormOptions( name, parent )
//
//////////////////////////////////////////////////////////////////////////////
function CfxWebFormOptions( name, parent )
{
	// Establish class and initialize instance.
	Cfx.Class.New( CfxWebFormOptions, CfxControl );
	this.InitInstance();
	
	// Set methods.
	this.Method( Initialize );
	this.Method( SubmitHandler );
	this.Method( Validate );
	this.Method( ValidateError );
	
	// Set properties.
	this.txtName = null;
	this.txtAddress1 = null;
	this.txtAddress2 = null;
	this.txtCity = null;
	this.selState = null;
	this.txtPostal = null;
	this.optHistory = null;
	this.lblError = null;
	
	// Initialize instance.
	if ( arguments.length )
	{
		this.name = name;
		this.parent = ( parent ) ? parent : Cfx.Dom.GetForm( 0 );
		this.items = this.GetItems( this.name, this.parent );
		this.Initialize();
	}

	//////////////////////////////////////////////////////////////////////////
	// Class methods definitions
	//////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////
	// Initialize
	//////////////////////////////////////////////////////////////////////////
	function Initialize()
	{
		this.txtName = new CfxText( "txtName", this, /^[a-zA-Z\s\.\,]+$/g );
		this.txtAddress1 = new CfxText( "txtAddress1", this );
		this.txtAddress2 = new CfxText( "txtAddress2", this );
		this.txtCity = new CfxText( "txtCity", this, /^[a-zA-Z\s\.]+$/g );
		this.selState = new CfxSelect( "selState", this );
		this.txtPostal = new CfxText( "txtPostal", this, /[{\d}5|{\d}5\-{\d}4]/g );
		this.optHistory = new CfxHistoryOptions( "optHistory", this );
		this.lblError = new CfxLabel( "lblError" , this );
		return true;
	}


	//////////////////////////////////////////////////////////////////////////
	//	SubmitHandler : Handles submission.
	//////////////////////////////////////////////////////////////////////////
	function SubmitHandler( handler )
	{
		this.GetElement( "btnSubmit" ).onclick = handler;
		this.txtName.SubmitHandler( handler );
		this.txtAddress1.SubmitHandler( handler );
		this.txtAddress2.SubmitHandler( handler );
		this.txtCity.SubmitHandler( handler );
		this.txtPostal.SubmitHandler( handler );
	}


	//////////////////////////////////////////////////////////////////////////
	//	Validate : Validate options.
	//////////////////////////////////////////////////////////////////////////
	function Validate()
	{
	    var retval = 1;
		var hasAddress = false;
		var isEmpty;

		// Turn off error indicators.
		this.txtName.ClearError();
		this.txtAddress1.ClearError();
		this.txtAddress2.ClearError();
		this.txtCity.ClearError();
		this.txtPostal.ClearError();
		this.lblError.ClearError();

		// Validate name field.
		if ( this.txtName.IsEmpty() )
		{
			return this.ValidateError( new CfxError( this.txtName, "Please enter user name" ) );
		}
		else
		{
			try
			{
				this.txtName.Validate()
			}
			catch ( cfxError )
			{
				return this.ValidateError( cfxError );
			}
		}
		
		// Validate address1 field.
		hasAddress = !this.txtAddress1.IsEmpty();

		// Validate address2 field.
		isEmpty = this.txtAddress2.IsEmpty();
		if ( !isEmpty && !hasAddress )
		{
			var cfxError = new CfxError ( this.txtAddress1, "Missing address line 1 information" );
			return this.ValidateError( cfxError );
		}

		// Validate city field.
		isEmpty = this.txtCity.IsEmpty();
		if ( isEmpty && hasAddress )
		{
			var cfxError = new CfxError( this.txtCity, "Missing city information" );
			return this.ValidateError( cfxError );
		}
		else if ( !isEmpty && !hasAddress )
		{
			var cfxError = new CfxError( this.txtAddress1, "Missing address line 1 information" );
			return this.ValidateError( cfxError );
		}
		else if ( !isEmpty && hasAddress )
		{
			try
			{ 
				retval = this.txtCity.Validate();
			}
			catch ( cfxError )
			{
				return this.ValidateError( cfxError );
			}
		}

		// Validate state field.
		isEmpty = ( this.selState.SelectedIndex() == 0 );
		if ( isEmpty && hasAddress )
		{
			var cfxError = new CfxError ( this.selState, "Please select a state" );
			return this.ValidateError( new CfxError ( this.selState, "Please select a state" ) );
		}
		else if ( !isEmpty && !hasAddress )
		{
			var cfxError = new CfxError ( this.txtAddress1, "Missing address line 1 information" );
			return this.ValidateError( cfxError );
		}
		else if ( !isEmpty && hasAddress )
			retval &= true;


		// Validate postal field.
		isEmpty = this.txtPostal.IsEmpty();
		if ( isEmpty && hasAddress )
		{
			var cfxError = new CfxError ( this.txtPostal, "Missing postal information" );
			return this.ValidateError( cfxError );
		}
		else if ( !isEmpty && !hasAddress )
		{
			var cfxError = new CfxError ( this.txtAddress1, "Missing address line 1 information" );
			return this.ValidateError( cfxError );
		}
		else if ( !isEmpty && hasAddress )
		{
			try 
			{
				retval = this.txtPostal.Validate();
			}
			catch ( cfxError )
			{
				return this.ValidateError( cfxError );
			}
		}

		// Validate history options field.
		retval &= this.optHistory.Validate();

		// Must return boolean value for postback resolution.
		return ( retval != 0 ) ? true : false;
	}

	//////////////////////////////////////////////////////////////////////////
	//	ValidateError
	//////////////////////////////////////////////////////////////////////////
	function ValidateError( cfxError )
	{
		cfxError.target.SetError();
		this.lblError.SetError( cfxError.message );
		return false;
	}
}
